# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.link import Link
from lusid.models.model_property import ModelProperty
from lusid.models.post_close_activity import PostCloseActivity
from lusid.models.version import Version

class ClosedPeriod(BaseModel):
    """
    ClosedPeriod
    """
    closed_period_id:  Optional[StrictStr] = Field(None,alias="closedPeriodId", description="The unique Id of the Closed Period. The ClosedPeriodId, together with the Timeline Scope and Code, uniquely identifies a Closed Period") 
    display_name:  Optional[StrictStr] = Field(None,alias="displayName", description="The name of the Closed Period.") 
    description:  Optional[StrictStr] = Field(None,alias="description", description="A description for the Closed Period.") 
    effective_start: Optional[datetime] = Field(default=None, description="The effective start of the Closed Period", alias="effectiveStart")
    effective_end: Optional[datetime] = Field(default=None, description="The effective end of the Closed Period", alias="effectiveEnd")
    as_at_closed: Optional[datetime] = Field(default=None, description="The asAt closed datetime for the Closed Period", alias="asAtClosed")
    properties: Optional[Dict[str, ModelProperty]] = Field(default=None, description="The Closed Periods properties. These will be from the 'ClosedPeriod' domain.")
    version: Optional[Version] = None
    post_close_activities: Optional[List[PostCloseActivity]] = Field(default=None, description="All the post close activities for the closed period.", alias="postCloseActivities")
    holdings_as_at_closed_override: Optional[datetime] = Field(default=None, description="The optional AsAtClosed Override to use for building holdings in the Closed Period.If not specified, the AsAtClosed on the Closed Period will be used.", alias="holdingsAsAtClosedOverride")
    valuation_as_at_closed_override: Optional[datetime] = Field(default=None, description="The optional AsAtClosed Override to use for performing valuations in the Closed Period.If not specified, the AsAtClosed on the Closed Period will be used.", alias="valuationAsAtClosedOverride")
    branch_status:  Optional[StrictStr] = Field(None,alias="branchStatus", description="The branch status of the closed period, e.g. Confirmed/Unconfirmed.") 
    href:  Optional[StrictStr] = Field(None,alias="href", description="The specific Uniform Resource Identifier (URI) for this resource at the requested asAt datetime.") 
    links: Optional[List[Link]] = None
    __properties = ["closedPeriodId", "displayName", "description", "effectiveStart", "effectiveEnd", "asAtClosed", "properties", "version", "postCloseActivities", "holdingsAsAtClosedOverride", "valuationAsAtClosedOverride", "branchStatus", "href", "links"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ClosedPeriod:
        """Create an instance of ClosedPeriod from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each value in properties (dict)
        _field_dict = {}
        if self.properties:
            for _key in self.properties:
                if self.properties[_key]:
                    _field_dict[_key] = self.properties[_key].to_dict()
            _dict['properties'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of version
        if self.version:
            _dict['version'] = self.version.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in post_close_activities (list)
        _items = []
        if self.post_close_activities:
            for _item in self.post_close_activities:
                if _item:
                    _items.append(_item.to_dict())
            _dict['postCloseActivities'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in links (list)
        _items = []
        if self.links:
            for _item in self.links:
                if _item:
                    _items.append(_item.to_dict())
            _dict['links'] = _items
        # set to None if closed_period_id (nullable) is None
        # and __fields_set__ contains the field
        if self.closed_period_id is None and "closed_period_id" in self.__fields_set__:
            _dict['closedPeriodId'] = None

        # set to None if display_name (nullable) is None
        # and __fields_set__ contains the field
        if self.display_name is None and "display_name" in self.__fields_set__:
            _dict['displayName'] = None

        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        # set to None if properties (nullable) is None
        # and __fields_set__ contains the field
        if self.properties is None and "properties" in self.__fields_set__:
            _dict['properties'] = None

        # set to None if post_close_activities (nullable) is None
        # and __fields_set__ contains the field
        if self.post_close_activities is None and "post_close_activities" in self.__fields_set__:
            _dict['postCloseActivities'] = None

        # set to None if holdings_as_at_closed_override (nullable) is None
        # and __fields_set__ contains the field
        if self.holdings_as_at_closed_override is None and "holdings_as_at_closed_override" in self.__fields_set__:
            _dict['holdingsAsAtClosedOverride'] = None

        # set to None if valuation_as_at_closed_override (nullable) is None
        # and __fields_set__ contains the field
        if self.valuation_as_at_closed_override is None and "valuation_as_at_closed_override" in self.__fields_set__:
            _dict['valuationAsAtClosedOverride'] = None

        # set to None if branch_status (nullable) is None
        # and __fields_set__ contains the field
        if self.branch_status is None and "branch_status" in self.__fields_set__:
            _dict['branchStatus'] = None

        # set to None if href (nullable) is None
        # and __fields_set__ contains the field
        if self.href is None and "href" in self.__fields_set__:
            _dict['href'] = None

        # set to None if links (nullable) is None
        # and __fields_set__ contains the field
        if self.links is None and "links" in self.__fields_set__:
            _dict['links'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ClosedPeriod:
        """Create an instance of ClosedPeriod from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ClosedPeriod.parse_obj(obj)

        _obj = ClosedPeriod.parse_obj({
            "closed_period_id": obj.get("closedPeriodId"),
            "display_name": obj.get("displayName"),
            "description": obj.get("description"),
            "effective_start": obj.get("effectiveStart"),
            "effective_end": obj.get("effectiveEnd"),
            "as_at_closed": obj.get("asAtClosed"),
            "properties": dict(
                (_k, ModelProperty.from_dict(_v))
                for _k, _v in obj.get("properties").items()
            )
            if obj.get("properties") is not None
            else None,
            "version": Version.from_dict(obj.get("version")) if obj.get("version") is not None else None,
            "post_close_activities": [PostCloseActivity.from_dict(_item) for _item in obj.get("postCloseActivities")] if obj.get("postCloseActivities") is not None else None,
            "holdings_as_at_closed_override": obj.get("holdingsAsAtClosedOverride"),
            "valuation_as_at_closed_override": obj.get("valuationAsAtClosedOverride"),
            "branch_status": obj.get("branchStatus"),
            "href": obj.get("href"),
            "links": [Link.from_dict(_item) for _item in obj.get("links")] if obj.get("links") is not None else None
        })
        return _obj

ClosedPeriod.update_forward_refs()
